/* -*-	Mode:C++; c-basic-offset:8; tab-width:8; indent-tabs-mode:t -*- */
/*
# Copyright (c) 2000-2001 by the University of Southern California.
# All rights reserved.
#
# Permission to use, copy, modify, and distribute this software and its
# documentation in source and binary forms for non-commercial purposes and
# without fee is hereby granted, provided that the above copyright notice
# appear in all copies and that both the copyright notice and this permission
# notice appear in supporting documentation, and that any documentation,
# advertising materials, and other materials related to such distribution and
# use acknowledge that the software was developed by the University of
# Southern California, Information Sciences Institute.  The name of the
# University may not be used to endorse or promote products derived from this
# software without specific prior written permission.
#
# THE UNIVERSITY OF SOUTHERN CALIFORNIA MAKES NO REPRESENTATIONS ABOUT THE
# SUITABILITY OF THIS SOFTWARE FOR ANY PURPOSE.  THIS SOFTWARE IS PROVIDED "AS
# IS" AND WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, WITHOUT
# LIMITATION, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
# PARTICULAR PURPOSE.
#
# Other copyrights might apply to parts of this software and are so noted when
# applicable.
# ----------------------------------------------------------------------------
#
# Primary author: Stephen Suryaputra
#                 September 1999
#This material is based upon work supported by the Defense Advanced Research
#Projects Agency Information Technology Office, (NGI Multicast Applications 
#and Architecture (NMAA-ADDON), ARPA Order No. H645, Program Code No. 9A20, 
#Issued by DARPA/CMD under Contract No. MDA972-99-C-0022.
*/
#include "config.h"
#include "packet.h"
#include "ip.h"
#include "classifier.h"
#include "swap.h"

class OpticalClassifier : public Classifier {
//Kamath: 06/19/2000
//Data for stats collection.
private: 
	int numPackets[max_num_of_wavelength];
protected:
	int classify(Packet *const p) {
		hdr_cmn *cmn = hdr_cmn::access(p);
		hdr_ip* iph = hdr_ip::access(p);

//Kamath: 06/13/2000
//Increment the hocpount.
		cmn->hopcount++;
//Update the data count.
		numPackets[iph->flowid()]++;	
		if (iph->flowid() == 0)
		{
			return (0);
		}
		cmn->switchedhopcount++;
		int x = cmn->iface();
//Kamath: Changed fro 16 bit shift to 8 bit shift: 16 bits caused a
//problem with large networks ( larger number of interfaces)
		x <<= 8;
//		x <<= 16;
	        x |= iph->flowid();
		return (x);
	}
	int command (int argc, const char*const* argv);
	void dumpStatistics();
public:
	OpticalClassifier()
	{
		for (int i = 0; i < max_num_of_wavelength; i++)
			numPackets[i] = -1;
	}

};

static class OpticalClassifierClass : public TclClass {
public:
	OpticalClassifierClass() : TclClass("Classifier/Optical") {}
	TclObject* create(int, const char*const*) {
		return (new OpticalClassifier());
	}
} class_optical_classifier;




int OpticalClassifier::command(int argc, const char*const* argv)
{
	
	Tcl& tcl = Tcl::instance();

	switch(argc) {
	case 2:
		if (strcmp(argv[1], "dump-stats") == 0) {
			dumpStatistics();
			return TCL_OK;
		}
		break;
	default:
		break;
	}

	Classifier::command(argc, argv);
}



void
OpticalClassifier::dumpStatistics()
{
	for(int count =0; count < max_num_of_wavelength; count++)
	{
		if(numPackets[count] > -1)
			printf("\tNumber of packets on wavelength %d= %d\n", count, numPackets[count]);
	}
}
