// -*-	Mode:C++; c-basic-offset:8; tab-width:8; indent-tabs-mode:t -*-
/*
# Copyright (c) 2000-2001 by the University of Southern California.
# All rights reserved.
#
# Permission to use, copy, modify, and distribute this software and its
# documentation in source and binary forms for non-commercial purposes and
# without fee is hereby granted, provided that the above copyright notice
# appear in all copies and that both the copyright notice and this permission
# notice appear in supporting documentation, and that any documentation,
# advertising materials, and other materials related to such distribution and
# use acknowledge that the software was developed by the University of
# Southern California, Information Sciences Institute.  The name of the
# University may not be used to endorse or promote products derived from this
# software without specific prior written permission.
#
# THE UNIVERSITY OF SOUTHERN CALIFORNIA MAKES NO REPRESENTATIONS ABOUT THE
# SUITABILITY OF THIS SOFTWARE FOR ANY PURPOSE.  THIS SOFTWARE IS PROVIDED "AS
# IS" AND WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, WITHOUT
# LIMITATION, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
# PARTICULAR PURPOSE.
#
# Other copyrights might apply to parts of this software and are so noted when
# applicable.
# ----------------------------------------------------------------------------
#
# Primary author: Stephen Suryaputra
#                 September 1999

#This material is based upon work supported by the Defense Advanced Research
#Projects Agency Information Technology Office, (NGI Multicast Applications 
#and Architecture (NMAA-ADDON), ARPA Order No. H645, Program Code No. 9A20, 
#Issued by DARPA/CMD under Contract No. MDA972-99-C-0022.
*/

#ifndef SWAP_H
#define SWAP_H
#include <assert.h>
#include <sys/types.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netinet/in.h>
#include "agent.h"

const double initial_backoff_period = 0.001;	// 1 ms
const int garbage_collect_cycle = 2;
const double garbage_collect_period =60; // 60 secs
//const int flow_table_bucket_size = 256;
const int flow_table_bucket_size = 1024; 
//const int max_num_of_wavelength = 512;
const int max_num_of_wavelength = 1024;
//const int max_num_of_wavelength_converter = 256;
const int max_num_of_wavelength_converter = 1024;
const int max_num_of_merging_device = 1024;
const int max_num_of_branch = 32;
const int med_str_len = 80;
const int big_str_len = 8192;
const int max_tcl_string = 127;
const int max_num_backoffs = 100;

#ifdef EGRESS
const int max_flows_on_egress = 1024;
#endif


// SWAP message types

enum SwapMessageType {
	SWAP_SETUP,
	SWAP_SETUP_CONFLICT,
        SWAP_SETUP_FAIL,
        SWAP_SETUP_EGRESS,
	SWAP_COMMIT,
	SWAP_COMMIT_OK,
	SWAP_TEARDOWN,
};

// SWAP message format

struct SwapMessage {
	SwapMessageType type;
	nsaddr_t ns_src;
//Kamath: 08/01/2000
//Egress switching modifications
	nsaddr_t ns_dst;
	struct in_addr ip_src;
	struct in_addr ip_dst;
	unsigned short tcp_sport;
	unsigned short tcp_dport;
	int lambda;
#ifdef EGRESS
	nsaddr_t egress;
#endif
	bool lambda_set[max_num_of_wavelength];


	static int offset_;
	inline static SwapMessage* access(Packet *p) {
		return (SwapMessage*) p->access(offset_);
	}
};

// taken from /usr/include/netinet/ip.h

struct ip {
	u_char	ip_vhl;			/* version << 4 | header length >> 2 */
	u_char	ip_tos;			/* type of service */
	u_short	ip_len;			/* total length */
	u_short	ip_id;			/* identification */
	u_short	ip_off;			/* fragment offset field */
	u_char	ip_ttl;			/* time to live */
	u_char	ip_p;			/* protocol */
	u_short	ip_sum;			/* checksum */
	struct	in_addr ip_src,ip_dst;	/* source and dest address */
};

//  taken from /usr/include/netinet/tcp.h

#define BYTE_ORDER LITTLE_ENDIAN

typedef	u_long	tcp_seq;
typedef u_long	tcp_cc;			/* connection count per rfc1644 */

struct tcphdr {
	u_short	th_sport;		/* source port */
	u_short	th_dport;		/* destination port */
	tcp_seq	th_seq;			/* sequence number */
	tcp_seq	th_ack;			/* acknowledgement number */
#if BYTE_ORDER == LITTLE_ENDIAN
	u_char	th_x2:4,		/* (unused) */
		th_off:4;		/* data offset */
#endif
#if BYTE_ORDER == BIG_ENDIAN
	u_char	th_off:4,		/* data offset */
		th_x2:4;		/* (unused) */
#endif
	u_char	th_flags;
#define	TH_FIN	0x01
#define	TH_SYN	0x02
#define	TH_RST	0x04
#define	TH_PUSH	0x08
#define	TH_ACK	0x10
#define	TH_URG	0x20
#define TH_FLAGS (TH_FIN|TH_SYN|TH_RST|TH_ACK|TH_URG)

	u_short	th_win;			/* window */
	u_short	th_sum;			/* checksum */
	u_short	th_urp;			/* urgent pointer */
};

extern struct in_addr inaddrany;

class Address;
class FlowEntry;
class LambdaEntry;
class InBranch;
class FlowAnalyzer;

#endif // SWAP_H

