/* -*-	Mode:C++; c-basic-offset:8; tab-width:8; indent-tabs-mode:t -*- */
/*
# Copyright (c) 2000-2001 by the University of Southern California.
# All rights reserved.
#
# Permission to use, copy, modify, and distribute this software and its
# documentation in source and binary forms for non-commercial purposes and
# without fee is hereby granted, provided that the above copyright notice
# appear in all copies and that both the copyright notice and this permission
# notice appear in supporting documentation, and that any documentation,
# advertising materials, and other materials related to such distribution and
# use acknowledge that the software was developed by the University of
# Southern California, Information Sciences Institute.  The name of the
# University may not be used to endorse or promote products derived from this
# software without specific prior written permission.
#
# THE UNIVERSITY OF SOUTHERN CALIFORNIA MAKES NO REPRESENTATIONS ABOUT THE
# SUITABILITY OF THIS SOFTWARE FOR ANY PURPOSE.  THIS SOFTWARE IS PROVIDED "AS
# IS" AND WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, WITHOUT
# LIMITATION, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
# PARTICULAR PURPOSE.
#
# Other copyrights might apply to parts of this software and are so noted when
# applicable.
# ----------------------------------------------------------------------------
#
# Primary author: Stephen Suryaputra
#                 September 1999
#This material is based upon work supported by the Defense Advanced Research
#Projects Agency Information Technology Office, (NGI Multicast Applications 
#and Architecture (NMAA-ADDON), ARPA Order No. H645, Program Code No. 9A20, 
#Issued by DARPA/CMD under Contract No. MDA972-99-C-0022.
*/

#include "net-interface.h"
#include "swap.h"
#include "swap_address.h"
#include "swap_analyzer.h"

char *
NsAddress::str()
{
	static char s[med_str_len];

	snprintf(s, med_str_len, "ns:%d", ns_addr_);
	return (s);
}

int
NsAddress::outIface(class FlowAnalyzer *analyzer)
{
	char *node = analyzer->node();
	Tcl& tcl = Tcl::instance();

	tcl.evalf("%s set out_dmux_", node);
	tcl.evalf("%s slot %d", tcl.result(), ns_addr_); // the link
	NetworkInterface *iface = (NetworkInterface *)
		tcl.lookup(tcl.result()); // the interface
	int label = iface->label();
	return (label);
}

char *
IpAddress::str()
{
	static char s[med_str_len];
	snprintf(s, med_str_len, "ip:%s", inet_ntoa(ip_addr_));
	return (s);
}

char *
TcpPort::str()
{
	static char s[med_str_len];

	snprintf(s, med_str_len, "tcp:%d", tcp_port_);
	return (s);
}

// Fine Addressing

char *
FineAddress::str()
{
	static char s[med_str_len];

	snprintf(s, med_str_len, "{%s,%s,%s}", NsAddress::str(),
		 IpAddress::str(), TcpPort::str());
	return (s);
}

inline unsigned int
FineAddress::val()
{
	return (tcp_port_ ^ ip_addr_.s_addr ^ ns_addr_);
}

void
FineAddress::fillSwapMessage(SwapMessage *swap)
{
	if (src_) {
		swap->ns_src = ns_addr_;
		swap->ip_src = ip_addr_;
		swap->tcp_sport = tcp_port_;
	} else {
		swap->ns_dst = ns_addr_;
		swap->ip_dst = ip_addr_;
		swap->tcp_dport = tcp_port_;
	}
}

bool
FineAddress::match(SwapAddress *a)
{
	if (type_ != a->type())
		return (false);
	FineAddress *fa = (FineAddress *)a;
	return (ns_addr_ == fa->nsAddr() &&
		ip_addr_.s_addr == fa->sAddr() &&
		tcp_port_ == fa->port());
}

// Medium Addressing

char *
MediumAddress::str()
{
	static char s[med_str_len];

	snprintf(s, med_str_len, "{%s,%s,*}", NsAddress::str(),
		 IpAddress::str());
	return (s);
}

inline unsigned int
MediumAddress::val()
{
	return(ip_addr_.s_addr ^ ns_addr_);
}

void
MediumAddress::fillSwapMessage(SwapMessage *swap)
{
	if (src_) {
		swap->ns_src = ns_addr_;
		swap->ip_src = ip_addr_;
	} else {
		swap->ns_dst = ns_addr_;
		swap->ip_dst = ip_addr_;
	}
}

bool
MediumAddress::match(SwapAddress *a)
{
	if (type_ != a->type())
		return (false);
	MediumAddress *ma = (MediumAddress *)a;
	return (ns_addr_ == ma->nsAddr() &&
		ip_addr_.s_addr == ma->sAddr());
}

// Coarse Addressing

char *
CoarseAddress::str()
{
	static char s[med_str_len];

	snprintf(s, med_str_len, "{%s,*,*}", NsAddress::str());
	return(s);
}

unsigned int
CoarseAddress::val()
{
	return(ns_addr_);
}

inline void
CoarseAddress::fillSwapMessage(SwapMessage *swap)
{
	if (src_)
		swap->ns_src = ns_addr_;
	else
		swap->ns_dst = ns_addr_;
}

bool
CoarseAddress::match(SwapAddress *a)
{
	if (type_ != a->type())
		return (false);
	CoarseAddress *ca = (CoarseAddress *)a;
	return (ns_addr_ == ca->nsAddr());
}


