
/*
# Copyright (c) 2000-2001 by the University of Southern California.
# All rights reserved.
#
# Permission to use, copy, modify, and distribute this software and its
# documentation in source and binary forms for non-commercial purposes and
# without fee is hereby granted, provided that the above copyright notice
# appear in all copies and that both the copyright notice and this permission
# notice appear in supporting documentation, and that any documentation,
# advertising materials, and other materials related to such distribution and
# use acknowledge that the software was developed by the University of
# Southern California, Information Sciences Institute.  The name of the
# University may not be used to endorse or promote products derived from this
# software without specific prior written permission.
#
# THE UNIVERSITY OF SOUTHERN CALIFORNIA MAKES NO REPRESENTATIONS ABOUT THE
# SUITABILITY OF THIS SOFTWARE FOR ANY PURPOSE.  THIS SOFTWARE IS PROVIDED "AS
# IS" AND WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, WITHOUT
# LIMITATION, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
# PARTICULAR PURPOSE.
#
# Other copyrights might apply to parts of this software and are so noted when
# applicable.
# ----------------------------------------------------------------------------
#
# Primary author: Stephen Suryaputra
#                 September 1999

#This material is based upon work supported by the Defense Advanced Research
#Projects Agency Information Technology Office, (NGI Multicast Applications 
#and Architecture (NMAA-ADDON), ARPA Order No. H645, Program Code No. 9A20, 
#Issued by DARPA/CMD under Contract No. MDA972-99-C-0022.
*/

#ifndef _SWAP_ADDRESS_H
#define _SWAP_ADDRESS_H
/* -*-	Mode:C++; c-basic-offset:8; tab-width:8; indent-tabs-mode:t -*- */

#include <netinet/in.h>
#include "swap.h"

enum AddrType {
	any_address,
        fine_address,
        medium_address,
        coarse_address
};
		
class SwapAddress {			// match any address
public:
	SwapAddress() { src_ = true; type_ = any_address; }
	void setAsDestination() { src_ = false; }
	AddrType type() { return type_; }
	virtual char *str() { return "*"; }
	virtual unsigned int val() { return 0; }
	virtual int outIface(FlowAnalyzer *analyzer) { return -1; }
	virtual bool match(SwapAddress *a) { return (type_ == a->type()); }
	virtual void fillSwapMessage(SwapMessage *swap) { }
protected:
	AddrType type_;		// XXX kludge
	bool src_;
};

class AddressElement {
public:
	virtual char *str() = 0;
};

class NsAddress : public AddressElement {
public:
	NsAddress(nsaddr_t ns_addr) {
		ns_addr_ = ns_addr;
	}
	virtual char *str();
	int outIface(FlowAnalyzer *analyzer);
	nsaddr_t nsAddr() { return ns_addr_; }
protected:
	nsaddr_t ns_addr_;
};

class IpAddress : public AddressElement {
public:
	IpAddress(struct in_addr ip_addr) {
		ip_addr_ = ip_addr;
	}
	virtual char *str();
	u_long sAddr() { return ip_addr_.s_addr; }
protected:
	struct in_addr ip_addr_;
};

class TcpPort : public AddressElement {
public:
	TcpPort(unsigned short tcp_port) {
		tcp_port_ = tcp_port;
	}
	virtual char *str();
	unsigned int port() { return tcp_port_; }
protected:
	unsigned short tcp_port_;
};

class FineAddress : public SwapAddress,
		    public NsAddress,
		    public IpAddress,
		    public TcpPort {
public:
	FineAddress(nsaddr_t ns_addr, struct in_addr ip_addr,
		    unsigned short tcp_port) :
		SwapAddress(),
		NsAddress(ns_addr),
		IpAddress(ip_addr),
		TcpPort(tcp_port)
	{
		type_ = fine_address;
	}
	virtual char *str();
	virtual unsigned int val();
	virtual int outIface(FlowAnalyzer *analyzer) {
		return (NsAddress::outIface(analyzer));
	}
	virtual void fillSwapMessage(SwapMessage *swap);
	virtual bool match(SwapAddress *a);
};

class MediumAddress : public SwapAddress,
		      public NsAddress,
		      public IpAddress {
public:
	MediumAddress(nsaddr_t ns_addr, struct in_addr ip_addr) :
		SwapAddress(),
		NsAddress(ns_addr),
		IpAddress(ip_addr) 
        {
		type_ = medium_address;
	}
	virtual char *str();
	virtual unsigned int val();
	virtual int outIface(FlowAnalyzer *analyzer) {
		return (NsAddress::outIface(analyzer));
	}
	virtual void fillSwapMessage(SwapMessage *swap);
	virtual bool match(SwapAddress *a);
};

class CoarseAddress : public SwapAddress, public NsAddress {
public:
	CoarseAddress(nsaddr_t ns_addr) :
		SwapAddress(),
		NsAddress(ns_addr)
	{
		type_ = coarse_address;
	}
	virtual char *str();
	virtual unsigned int val();
	virtual int outIface(FlowAnalyzer *analyzer) {
		return (NsAddress::outIface(analyzer));
	}
	virtual void fillSwapMessage(SwapMessage *swap);
	virtual bool match(SwapAddress *a);
};


#endif




