/* -*-	Mode:C++; c-basic-offset:8; tab-width:8; indent-tabs-mode:t -*- */
/*
# Copyright (c) 2000-2001 by the University of Southern California.
# All rights reserved.
#
# Permission to use, copy, modify, and distribute this software and its
# documentation in source and binary forms for non-commercial purposes and
# without fee is hereby granted, provided that the above copyright notice
# appear in all copies and that both the copyright notice and this permission
# notice appear in supporting documentation, and that any documentation,
# advertising materials, and other materials related to such distribution and
# use acknowledge that the software was developed by the University of
# Southern California, Information Sciences Institute.  The name of the
# University may not be used to endorse or promote products derived from this
# software without specific prior written permission.
#
# THE UNIVERSITY OF SOUTHERN CALIFORNIA MAKES NO REPRESENTATIONS ABOUT THE
# SUITABILITY OF THIS SOFTWARE FOR ANY PURPOSE.  THIS SOFTWARE IS PROVIDED "AS
# IS" AND WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, WITHOUT
# LIMITATION, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
# PARTICULAR PURPOSE.
#
# Other copyrights might apply to parts of this software and are so noted when
# applicable.
# ----------------------------------------------------------------------------
#
# Primary author: Stephen Suryaputra
#                 September 1999

#This material is based upon work supported by the Defense Advanced Research
#Projects Agency Information Technology Office, (NGI Multicast Applications 
#and Architecture (NMAA-ADDON), ARPA Order No. H645, Program Code No. 9A20, 
#Issued by DARPA/CMD under Contract No. MDA972-99-C-0022.
*/

#ifndef SWAP_ENTRY_H
#define SWAP_ENTRY_H
#include "swap.h"
#include "swap_address.h"
#include "flow_converter.h"
#include "merging_device.h"

enum InState {
        IN_PASSIVE,
	IN_ACTIVE,
	IN_WAIT_COMMIT,
	IN_SWITCHED,
	IN_SETUP_SENT,
	IN_BACKOFF,
	IN_WAIT_OUT_SWITCHED,
};    

enum OutState {
        OUT_NOT_SWITCHED,
        OUT_COMMIT_SENT,
        OUT_ACTIVE_SWITCHED,
        OUT_ABOUT_TO_TEARDOWN,
        OUT_WAIT_ALL_COMMIT,
        OUT_WAIT_COMMIT_OK,
        OUT_SWITCHED,
};

class FlowTimer : public TimerHandler {
public:
	FlowTimer(FlowEntry *entry) : TimerHandler() {
		entry_ = entry;
		active_ = false;
	}
	bool& active() { return active_; }
protected:
        virtual void expire(Event *e);
	FlowEntry *entry_;
	bool active_;
};

class FlowEntry {
public:
	FlowEntry(FlowAnalyzer *analyzer, SwapAddress *src, SwapAddress *dst);
	~FlowEntry();
	void clearStats() { pkt_cnt_ = 0; pkt_switched_cnt_ = 0; }
	FlowAnalyzer* analyzer() { return analyzer_; }
	int& pktCnt() { return pkt_cnt_; }
	OutState state() { return state_; }
	void newState(OutState new_state);
	int& outLambda() { return out_lambda_; }
	int& buck() { return buck_; }
	FlowEntry* & next() { return next_; }
	FlowEntry* & prev() { return prev_; }
	InBranch *branch() { return branch_; }
	SwapAddress *src() { return src_; }
	SwapAddress *dst() { return dst_; }
	int linkIndex() { return link_idx_; }
	FlowConverter* & converter() { return wcp_; }
	MergingDevice* & mergingDevice() { return merging_device_; }
	void startTimer(double expire);
	void stopTimer();
	char *flowStr();
	bool hasOutstandingBranch();
	bool hasDownstreamNeighbor();
	void insertBranch(InBranch *branch);
	void removeBranch(InBranch *branch);
	InBranch *searchBranch(Packet *p);
	void constructSwapMessage(SwapMessage *swap);
protected:
	OutState state_;




	int pkt_cnt_;
	int pkt_switched_cnt_;
	int out_lambda_;
	int buck_;
	int link_idx_;
	FlowAnalyzer *analyzer_;
	FlowEntry *next_;
	FlowEntry *prev_;
	FlowTimer *timer_;
	InBranch *branch_;
	SwapAddress *src_;
	SwapAddress *dst_;
	FlowConverter *wcp_;	// XXX think later
	MergingDevice *merging_device_;
};

class BranchTimer : public TimerHandler {
public:
	BranchTimer(InBranch *branch) : TimerHandler() {
		branch_ = branch;
		active_ = false;
	}
	bool& active() { return active_; }
        virtual void expire(Event *e);
protected:
	InBranch *branch_;
	bool active_;
};

class InBranch {
public:
	InBranch(FlowEntry *entry, int link_idx);
	~InBranch();
	int& pktCnt() { return pkt_cnt_; }
	int& inLambda() { return in_lambda_; }
	InBranch* & next() { return next_; }
	FlowEntry *entry() { return entry_; }
	InState state() { return state_; }
	int& backoffCount() { return backoff_count_; }
	void newState(InState new_state);
	int linkIndex() { return link_idx_; }
	int& garbageCnt() { return garbage_cnt_; }
	void startTimer(double expire);

//Kamath: 06/13/2000
//Added stopTimer to change the X/Y classifier model.
	void stopTimer();
	bool hasUpstreamNeighbor();
	bool *lambdaSet();
	bool isLocked();
	bool hasAvailableLambda();
	void backoff(bool in_timer);
	void activate();
	void deactivate();
	BranchTimer *timer_;
protected:
	int in_lambda_;
	int pkt_cnt_;
	InState state_;
	int backoff_count_;
	int link_idx_;
	FlowEntry *entry_;
	InBranch *next_;
	int garbage_cnt_;
};


#ifdef EGRESS

class LambdaEntry {
public:
	LambdaEntry();
	~LambdaEntry();
	int& pktCnt() { return pkt_cnt_; }
	nsaddr_t & egressNode() { return egress; }
	//void startTimer(double expire);
	//void stopTimer();
	int flowcount;
	FlowEntry *entry[max_flows_on_egress];
protected:
	int pkt_cnt_;
	nsaddr_t egress;
};
#endif

#endif // SWAP_ENTRY_H
