/* -*-	Mode:C++; c-basic-offset:8; tab-width:8; indent-tabs-mode:t -*- */
/*
 * Copyright (c) Xerox Corporation 1997. All rights reserved.
 *  
 * License is granted to copy, to use, and to make and to use derivative
 * works for research and evaluation purposes, provided that Xerox is
 * acknowledged in all documentation pertaining to any such copy or derivative
 * work. Xerox grants no other licenses expressed or implied. The Xerox trade
 * name should not be used in any advertising without its written permission.
 *  
 * XEROX CORPORATION MAKES NO REPRESENTATIONS CONCERNING EITHER THE
 * MERCHANTABILITY OF THIS SOFTWARE OR THE SUITABILITY OF THIS SOFTWARE
 * FOR ANY PARTICULAR PURPOSE.  The software is provided "as is" without
 * express or implied warranty of any kind.
 *  
 * These notices must be retained in any copies of any part of this software.
 */

/*
 * Copyright (c) 1997, 1998 The Regents of the University of California.
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 * 	This product includes software developed by the MASH Research
 *	Group at the University of California, Berkeley.
 * 4. Neither the name of the University nor of the Research Group may be used
 *    to endorse or promote products derived from this software without
 *    specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */
/*
# Copyright (c) 2000-2001 by the University of Southern California.
# All rights reserved.
#
# Permission to use, copy, modify, and distribute this software and its
# documentation in source and binary forms for non-commercial purposes and
# without fee is hereby granted, provided that the above copyright notice
# appear in all copies and that both the copyright notice and this permission
# notice appear in supporting documentation, and that any documentation,
# advertising materials, and other materials related to such distribution and
# use acknowledge that the software was developed by the University of
# Southern California, Information Sciences Institute.  The name of the
# University may not be used to endorse or promote products derived from this
# software without specific prior written permission.
#
# THE UNIVERSITY OF SOUTHERN CALIFORNIA MAKES NO REPRESENTATIONS ABOUT THE
# SUITABILITY OF THIS SOFTWARE FOR ANY PURPOSE.  THIS SOFTWARE IS PROVIDED "AS
# IS" AND WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, WITHOUT
# LIMITATION, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
# PARTICULAR PURPOSE.
#
# Other copyrights might apply to parts of this software and are so noted when
# applicable.
# ----------------------------------------------------------------------------
#
# Primary author: Purushotham Kamath
#                 April 2000

#This material is based upon work supported by the Defense Advanced Research
#Projects Agency Information Technology Office, (NGI Multicast Applications 
#and Architecture (NMAA-ADDON), ARPA Order No. H645, Program Code No. 9A20, 
#Issued by DARPA/CMD under Contract No. MDA972-99-C-0022.
*/



#ifndef lint
static const char rcsid[] =
    "@(#) $Header: /auto/boreas/jade/pkamath/cvsroot/pow/ns-2.1b6/trafgentcpdump.cc,v 1.3 2001/04/19 21:18:04 pkamath Exp $ (Xerox)";
#endif

/* XXX: have not dealt with errors.  e.g., if something fails during
 * TraceFile::setup(), or TrafficTrace is not pointing to a TraceFile,
 * no guarantee about results.
 */

//Utham Kamath: Code changes based on code from traffictrace.cc 
//emulate/tap.cc and emulate/net-pcap.cc

#include <sys/types.h>
#include <sys/stat.h> 
#include <stdio.h>

#include "config.h"
#ifdef HAVE_NETINET_IN_H
#include <netinet/in.h>
#endif /* HAVE_NETINET_IN_H */


#include "random.h"
/* module to implement a traffic generator in ns driven by a trace
 * file.  records in the trace file consist of 2 32 bit fields, the
 * first indicating the inter-packet time in microseconds, and the
 * second indicating the packet length in bytes.  multiple TraffficTrace
 * objects can use the same trace file, and different TrafficTrace
 * objects can use different trace files.  For each TrafficTrace, a
 * random starting point within the trace file is selected.
 */

#include "object.h"
#include "trafgen.h"
#include <emulate/net.h>
#include "ip.h"
#include <sys/types.h>
//#include <netine"t/in.h>"
#include <netinet/in_systm.h>
#include <netinet/ip.h>


class TrafficGeneratorTcpDump : public TrafficGenerator {
 public:
	TrafficGeneratorTcpDump();
	int command(int argc, const char*const* argv);
	virtual double next_interval(int &);
 protected:
	void start();
	void timeout();
	Network *net_;
	Packet *pkt;	
	int maxpkt_;
	void init();
};


static class TrafficGeneratorTcpDumpClass : public TclClass {
 public:
	TrafficGeneratorTcpDumpClass() : TclClass("Application/Traffic/TcpDump") {}
	TclObject* create(int, const char*const*) {
	        return(new TrafficGeneratorTcpDump());
	}
} class_traffictrace;

TrafficGeneratorTcpDump::TrafficGeneratorTcpDump(): net_(NULL), pkt(NULL)
{
}

void TrafficGeneratorTcpDump::init()
{
}

int TrafficGeneratorTcpDump::command(int argc, const char*const* argv)
{
	Tcl& tcl = Tcl::instance();
	
	if (argc == 3) {
		if (strcmp(argv[1], "attach-tracefile") == 0) {
			net_ = (Network *)TclObject::lookup(argv[2]);
			if (net_ != 0) {
				return TCL_OK;
			} else {
			    	return (TCL_ERROR);
			}
		    return(TCL_OK);
		}
	}

	return (TrafficGenerator::command(argc, argv));

}

void TrafficGeneratorTcpDump::timeout()
{
        if (! running_)
                return;

        /* send a packet */
	// Note:  May need to set "NEW_BURST" flag in sendmsg() for 
	// signifying a new talkspurt when using vat traces.
	// (see expoo.cc, tcl/ex/test-rcvr.tcl)

//Utham Kamath: Tell the agent to send the packet.
        agent_->send(pkt, NULL);

        /* figure out when to send the next one */
        nextPkttime_ = next_interval(size_);
        /* schedule it */
        timer_.resched(nextPkttime_);
}

void TrafficGeneratorTcpDump::start()
{

	init();
	running_ = 1;
	nextPkttime_ = next_interval(size_);

	if(nextPkttime_ < 0)
	{
			running_ = 0;
			return;
	}

	timer_.resched(nextPkttime_);
	// Enable line below if you want to send immediately upon start
	//timeout(); 

}

double TrafficGeneratorTcpDump::next_interval(int& size)
{

//Utham Kamath: Call the net-pcap-file's recv instead of the tfile's 
//get_next function.
	double ts =0;
	sockaddr addr;			//currently not used.
	pkt = agent_->allocpkt(1600);
  size = net_->recv(pkt->accessdata(), 1600, addr, ts);
	if(size < 0)
	{
		running_ = 0;
		stop();
		return 0;
	}
//Kamath: 06/11/2000
//The ns packet sizes did not reflect the IP packet sizes, thereby having an 
//effect on the simulation.
	hdr_cmn* ch = hdr_cmn::access(pkt);
	hdr_ip* iph = hdr_ip::access(pkt);
	struct ip *ip = (struct ip *) pkt->accessdata();
	ch->size() = size + ip->ip_len;
	iph->flowid() = 0;
//Kamath: 06/13/2000
//Setting the hopcounts to 0. 
	ch->hopcount = 0;
	ch->iphopcount = 0;
	ch->switchedhopcount = 0;
	//EOF


	return(ts - Scheduler::instance().clock()); 

}
