# Copyright (c) 2000-2001 by the University of Southern California.
# All rights reserved.
#
# Permission to use, copy, modify, and distribute this software and its
# documentation in source and binary forms for non-commercial purposes and
# without fee is hereby granted, provided that the above copyright notice
# appear in all copies and that both the copyright notice and this permission
# notice appear in supporting documentation, and that any documentation,
# advertising materials, and other materials related to such distribution and
# use acknowledge that the software was developed by the University of
# Southern California, Information Sciences Institute.  The name of the
# University may not be used to endorse or promote products derived from this
# software without specific prior written permission.
#
# THE UNIVERSITY OF SOUTHERN CALIFORNIA MAKES NO REPRESENTATIONS ABOUT THE
# SUITABILITY OF THIS SOFTWARE FOR ANY PURPOSE.  THIS SOFTWARE IS PROVIDED "AS
# IS" AND WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, WITHOUT
# LIMITATION, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
# PARTICULAR PURPOSE.
#
# Other copyrights might apply to parts of this software and are so noted when
# applicable.
# ----------------------------------------------------------------------------
#
# Primary author: Stephen Suryaputra
#                 September 1999

#This material is based upon work supported by the Defense Advanced Research
#Projects Agency Information Technology Office, (NGI Multicast Applications 
#and Architecture (NMAA-ADDON), ARPA Order No. H645, Program Code No. 9A20, 
#Issued by DARPA/CMD under Contract No. MDA972-99-C-0022.



Agent/SWAP set packetSize_ 64 
#
# Flow-id Routing Nodes:
# like normal nodes, but with a hash classifier.
#

# a dump switch which switch packets based only on the flow_id
Class SwiftNode -superclass Node
SwiftNode set num_of_flow_converter_ 512 
#SwiftNode set use_converter_ 0
SwiftNode instproc mk-default-classifier {} {
    $self instvar address_ classifier_ id_ dmux_ addr_classifier_ \
		ip_router_ flow_analyzer_ out_dmux_


#The optical classifier
#Flowid 0 packets go to ip_router_ ( Non flow packets) 
#All other packets go out the router (?) 
    set classifier_ [new Classifier/Optical 500]

#A classifier to switch packets either to
#the FlowAnalyzer or to the SWAP Agent.
    set ip_router_ [new Classifier/Addr]
    $ip_router_ set mask_ [AddrParams set NodeMask_(1)]
    $ip_router_ set shift_ [AddrParams set NodeShift_(1)]
    
#Setting the opticalclassifier's target. 
    set speed 620Mb
    set delay 0.25ms
    set target [$self make-internal-link-to $ip_router_ $speed $delay]
    set slot [$classifier_ installNext $target]

#Create a Flow Analyzer. 
    set flow_analyzer_ [new FlowAnalyzer/Medium]
    $flow_analyzer_ flow_detect_time 20 
    $flow_analyzer_ flow_active_time 10 
    $flow_analyzer_ high-threshold 10 
    $flow_analyzer_ low-threshold  5 
    $flow_analyzer_ backoff-period 1ms 
    $flow_analyzer_ backoff-limit 10 
    $flow_analyzer_ num-of-wavelength 4 
    $flow_analyzer_ node $self
    $flow_analyzer_ node-str $self


#Setting the optical classifer and the ip_router_'s target.
    $classifier_ defaulttarget $ip_router_
    $ip_router_ defaulttarget $flow_analyzer_

#Output classifer
    set out_dmux_ [new Classifier/Addr]
    $out_dmux_ set mask_ [AddrParams set NodeMask_(1)]
    $out_dmux_ set shift_ [AddrParams set NodeShift_(1)]

#Setting the Flow Anlyzer's target.
    set dnlink [$self make-internal-link-to $out_dmux_ $speed 0ms]
    $flow_analyzer_ target $dnlink

#Flow converter
    $self instvar converter
    set max [SwiftNode set num_of_flow_converter_]
    for {set i 0} {$i < $max} {incr i} {
	set converter [new FlowConverter]
	$flow_analyzer_ wavelength-converter $converter
    }
    set address_ $id_


# 
# When an agent is created,
# $self add-route $address_ $dmux_ is called
# which will do this.
#


}

SwiftNode instproc add-route { dst target } {
    $self instvar ip_router_ flow_analyzer_ address_ out_dmux_
    if {$dst == $address_} {
	$ip_router_ install $dst $target
    } else {
	$out_dmux_ install $dst $target
    }
}

SwiftNode instproc make-internal-link-to {target bw delay} {
    set q [new Queue/DropTail]
    set link [new DelayLink]
    $link set bandwidth_ $bw
    $link set delay_ $delay
    $link target $target
    $q target $link
    return $q
}

SwiftNode instproc attach { agent } {
    $self instvar out_dmux_ id_
    eval $self next $agent

    # set agent to send to out_dmux instead of entry
    $agent target $out_dmux_
}

SwiftNode instproc dump-stats {} {
    $self instvar flow_analyzer_ id_ classifier_
    $flow_analyzer_ dump-stats
    $classifier_ dump-stats

}





