/*
# Copyright (c) 2000-2001 by the University of Southern California.
# All rights reserved.
#
# Permission to use, copy, modify, and distribute this software and its
# documentation in source and binary forms for non-commercial purposes and
# without fee is hereby granted, provided that the above copyright notice
# appear in all copies and that both the copyright notice and this permission
# notice appear in supporting documentation, and that any documentation,
# advertising materials, and other materials related to such distribution and
# use acknowledge that the software was developed by the University of
# Southern California, Information Sciences Institute.  The name of the
# University may not be used to endorse or promote products derived from this
# software without specific prior written permission.
#
# THE UNIVERSITY OF SOUTHERN CALIFORNIA MAKES NO REPRESENTATIONS ABOUT THE
# SUITABILITY OF THIS SOFTWARE FOR ANY PURPOSE.  THIS SOFTWARE IS PROVIDED "AS
# IS" AND WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, WITHOUT
# LIMITATION, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
# PARTICULAR PURPOSE.
#
# Other copyrights might apply to parts of this software and are so noted when
# applicable.
# ----------------------------------------------------------------------------
#
# Primary author: Purushotham Kamath
#                 April 2000

#This material is based upon work supported by the Defense Advanced Research
#Projects Agency Information Technology Office, (NGI Multicast Applications 
#and Architecture (NMAA-ADDON), ARPA Order No. H645, Program Code No. 9A20, 
#Issued by DARPA/CMD under Contract No. MDA972-99-C-0022.
*/


#include <stdio.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <pcap.h>
#include <net/bpf.h>
#include <netinet/in_systm.h>
#include <netinet/in.h>
#define _IP_VHL
#include <netinet/ip.h>
#include <netinet/tcp.h>
#include <arpa/inet.h>
//#include <../../ns-2.1b6/in_cksum.h>

#define TCPDUMP_MAGIC           0xa1b2c3d4  /* tcpdump magic number */
#ifndef MICROSEC
#define MICROSEC                1000000
#endif

int main(int argc, char **argv)
{
struct pcap_file_header pcfh;
struct pcap_pkthdr p;
struct ip iph;
struct tcphdr tcph;
FILE *in_fp, *fp2, *out_fp;
u_short id = 0;
u_long time;
u_short len;
u_int   src_addr, dst_addr;
u_short sport, dport;

u_int f1, f2, f3, f4;
char field1[20];
char field2[20];
char field3[20];
char field4[20];
char field5[20];
char field6[20];
char field7[20];
char field8[20];
char field9[20];
char field10[20];
double ts;
	int secs =0;
	int usecs =0;

	if (argc != 3) 
	{
		printf(stderr, "Usage: %s input_filename output_filename\n\nUse - for stdin and/or stdout.\n", argv[0]);
		exit(1);
 	}
	
    	if (argv[1][0] == '-' && argv[1][1] == '\0')
		in_fp = stdin;
 	 else 
	 {
      		in_fp = fopen(argv[1], "r");

		if (in_fp == NULL) 
		{
			fprintf(stderr, "fopen error opening file %s for reading.\n", argv[1]);
      			exit(1);
  		}
    	}

 	if (argv[2][0] == '-' && argv[2][1] ==   '\0')
		out_fp = stdout;
	else 
	{
      		out_fp = fopen(argv[2], "w");
	 	if (out_fp == NULL) 
		{
			fprintf(stderr, "fopen error opening file %s for writing.\n", argv[2]);
			exit(1);
		}
	}

	/* write libpcap file header */
	pcfh.magic = TCPDUMP_MAGIC;
	pcfh.version_major = PCAP_VERSION_MAJOR;
	pcfh.version_minor = PCAP_VERSION_MINOR;
	pcfh.thiszone = 0;                /* GMT to local time correction */



	pcfh.sigfigs = 0;                 /* accuracy of timestamps */
	pcfh.snaplen = 40;                /* max length saved portion of each
  pkt */
    	pcfh.linktype = DLT_NULL;  /* datalink == NULL encapsulated */

      	if (fwrite((char *)&pcfh, sizeof(pcfh), 1, out_fp) != 1) {
		fprintf(stderr, "error writing pcap file header to file %s\n", argv[2]);
	     	 exit (1);
	}	

	while (!feof(in_fp)) {
	/* read the ASCII trace */

	if (fscanf(in_fp, "%s%s%s%s%s%s%s%s%s%s\n", field1, field2, field3, field4, field5, field6, field7, field8, field9, field10) == 0) {
		fprintf(stderr, "error reading input file\n");
		exit (1);
	}

	sscanf(field1, "%lf", &ts);
        secs = abs(ts);
	usecs = (ts - abs(ts)) * 1000000; 


//	sscanf(field1, "%d.%d", &secs, &usecs);
	sscanf( field7, "%u.%u.%u.%u", &f1, &f2, &f3, &f4);
	src_addr = ((((f1 << 24) +(f2 << 16))+(f3 << 8) ) + f4);
	sscanf( field8, "%u.%u.%u.%u", &f1, &f2, &f3, &f4);
	dst_addr = ((((f1 << 24) +(f2 << 16))+(f3 << 8) ) + f4);
	sport = atol(field9); 
	dport= atol(field10);
	len=atol(field5, NULL, 10);



	/* write the pcap packet header */
	p.ts.tv_sec = secs;
//	(long) (time/25/MICROSEC);

//AUCK2 format is up to 8 decimal places.
//	p.ts.tv_usec = usecs/100;


//	(long) ((time/25) - (p.ts.tv_sec * MICROSEC));
//	p.ts.tv_sec += 18000;     /*  kludge - should set thiszone properly*/

	p.caplen = 40; 
      	p.len = len;
//	printf("Writing.\n");
//	fflush(stdout);


	if (fwrite((char *)&p, sizeof(p), 1, out_fp) != 1) {
		fprintf(stderr, "error writing pcap packet header to output file\n");
		exit (1);
	}

	/* write IP header */
//	iph.ip_vhl = IP_VHL_BORING;
	iph.ip_tos = 0;
	iph.ip_len = len;
	iph.ip_id = ++id;
        iph.ip_off = 0;




	iph.ip_ttl = 255;
      	iph.ip_p = IPPROTO_TCP;
	iph.ip_src.s_addr = src_addr;
	iph.ip_dst.s_addr = dst_addr;
//	iph.ip_sum = in_cksum_hdr(&iph);

	if (fwrite((char *)&iph, sizeof(iph), 1, out_fp) != 1) {
		fprintf(stderr, "error writing packet to output file\n");
	      	exit (1);
	}

	 /* write TCP header */
	tcph.th_sport = sport;
	tcph.th_dport = dport;
	tcph.th_seq = 0;          /* we don't care about this anyway for now
									    */
	tcph.th_ack = 0;
	tcph.th_off = 0;





	tcph.th_flags = 0;
	tcph.th_win = 0;
	tcph.th_urp = 0;
	/*      tcph.th_sum */

	if (fwrite((char *)&tcph, sizeof(tcph), 1, out_fp) != 1) {
		fprintf(stderr, "error writing packet to output file\n");
		exit (1);
	}
	} /* while */

	fclose(in_fp);
	fclose(out_fp);
	return(0);




}











