/*
# Copyright (c) 2000-2001 by the University of Southern California.
# All rights reserved.
#
# Permission to use, copy, modify, and distribute this software and its
# documentation in source and binary forms for non-commercial purposes and
# without fee is hereby granted, provided that the above copyright notice
# appear in all copies and that both the copyright notice and this permission
# notice appear in supporting documentation, and that any documentation,
# advertising materials, and other materials related to such distribution and
# use acknowledge that the software was developed by the University of
# Southern California, Information Sciences Institute.  The name of the
# University may not be used to endorse or promote products derived from this
# software without specific prior written permission.
#
# THE UNIVERSITY OF SOUTHERN CALIFORNIA MAKES NO REPRESENTATIONS ABOUT THE
# SUITABILITY OF THIS SOFTWARE FOR ANY PURPOSE.  THIS SOFTWARE IS PROVIDED "AS
# IS" AND WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, WITHOUT
# LIMITATION, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
# PARTICULAR PURPOSE.
#
# Other copyrights might apply to parts of this software and are so noted when
# applicable.
# ----------------------------------------------------------------------------
#
# Primary author: Purushotham Kamath
#                 April 2000

This material is based upon work supported by the Defense Advanced Research
Projects Agency Information Technology Office, (NGI Multicast Applications 
and Architecture (NMAA-ADDON), ARPA Order No. H645, Program Code No. 9A20, 
Issued by DARPA/CMD under Contract No. MDA972-99-C-0022.
*/

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <sys/types.h>

FILE * in_fp;
FILE * in_fp2;
FILE * in_fp3;

long seed = 010273;


class DstInfo
{
public:
	DstInfo()
	{
		probability =0;
		dstipcount=0;
//		dstip_set = new char [20000][20];
	}
	FILE *fout;
	char dst[20];
	float probability;
	char dstip_set[19000][15];
	int dstipcount;
};


int main(int argc, char **argv)
{
	char vbnssrc[20];
	DstInfo vbnsdstinfo[200];


	srandom(seed);
	int numNodes=0;


	if (argc != 4) 
	{
		fprintf(stderr, "Usage: %s srcnode dstlistfile tracefile\n", argv[0]);
		exit(1);
 	}

	char filename[39];
	sprintf(filename, "./vbns-%s", argv[1]);

      	in_fp = fopen(filename, "r");
	if (in_fp == NULL) 
	{
		fprintf(stderr, "fopen error opening file %s for reading.\n", argv[1]);
      		exit(1);
  	}

      	in_fp2 = fopen(argv[2], "r");
	if (in_fp2 == NULL) 
	{
		fprintf(stderr, "fopen error opening file %s for reading.\n", argv[2]);
      		exit(1);
  	}
      	
	in_fp3 = fopen(argv[3], "r");
	if (in_fp3 == NULL) 
	{
		fprintf(stderr, "fopen error opening file %s for reading.\n", argv[3]);
      		exit(1);
  	}

	fflush(stdout);

	char dummy[20];
	char dummy2[20];
	while (!feof(in_fp)) 
	{

		if (fscanf(in_fp, "%s%s%s%s\n", vbnssrc ,dummy, vbnsdstinfo[numNodes].dst, dummy2) ==0)
		
		{
			fprintf(stderr, "error reading input file\n");
			exit (1);
		}
		fflush(stdout);
		vbnsdstinfo[numNodes].probability = strtod(dummy2, NULL);
		sprintf(filename, "lbl-pkt-5.trc.%s.%s", vbnssrc, vbnsdstinfo[numNodes].dst);
		vbnsdstinfo[numNodes].fout = fopen(filename, "w");
		numNodes++;
	}



	char dstip[20];
	while (!feof(in_fp2)) 
	{

		if (fscanf(in_fp2, "%s\n", dstip) ==0) 
		{
			fprintf(stderr, "error reading input file\n");
			exit (1);
		}
	
		
		float probSum=0;
		float rnd = random();
		float srnd = rnd/(pow(2,31)-1);
		for (int count =0; count < numNodes; count++)
		{
			probSum+=vbnsdstinfo[count].probability;
			fflush(stdout);
			if(srnd < probSum)
			{
				strcpy(vbnsdstinfo[count].dstip_set[vbnsdstinfo[count].dstipcount] , dstip);
				vbnsdstinfo[count].dstipcount++;
				break;
			}
		}
	}

	 char dummy1[20];
	 char dummy3[20];
	 char dummy4[20];
	 char dummy5[20];
	 char dummy6[20];
	 char dummy7[20];
	 char dummy8[20];


	while (!feof(in_fp3))
	{













		if (fscanf(in_fp3, "%s%s%s%s%s%s\n", dummy1 ,dummy2, dstip, dummy3, dummy4, dummy5) ==0)
		{
			fprintf(stderr, "error reading input file\n");
			exit (1);
		}


//Put in a dummy line so that all trace files begin with the same initial time.
		static bool firstrun = true;
		if(firstrun)
		{
			for(int count = 0; count < numNodes; count++)
			{
				fprintf( vbnsdstinfo[count].fout, "%s %s %s %s %s %s\n", dummy1, "0", "0" , "0", "0", "0");
			}
			firstrun = false;
		}





		for(int count=0; count <numNodes; count++)
		{
			for( int count2 =0; count2 < vbnsdstinfo[count].dstipcount; count2++)
			{
				if(strcmp(dstip, vbnsdstinfo[count].dstip_set[count2]) == 0)
				{
					fprintf(vbnsdstinfo[count].fout, "%s %s %s %s %s %s\n", dummy1 ,dummy2, dstip, dummy3, dummy4, dummy5);
				}
			}
		}
	}


	fclose(in_fp);
	fclose(in_fp2);
	fclose(in_fp3);
	return(0);




}











